<?php

/**
 * @file
 * CivicActions's Patch Page.
 */

function civicactions_patch_page() {
  if (module_exists('update')) {
    $patches = array();
    // Retrieve the project information from the update_status module.
    module_load_include('inc', 'update', 'update.compare');
    $projects = update_get_projects();
    // Find the Drupal core patches.
    foreach (file_scan_directory("./", ".patch$", array('.', '..', 'CVS', 'sites')) as $file) {
        if (!isset($patches['drupal'])) {
          $patches['drupal'] = array('project' => $projects['drupal'], 'files' => array());
        }
        $patches['drupal']['files'][] = $file;
    }
    // Find the contrib patches.
    $dirs = array();
    foreach ($projects as $project) {
      if ($project['name'] == 'drupal') {
        continue;
      }
      if ($path = _civicactions_get_project_path($project)) {
        foreach (file_scan_directory($path, ".patch$") as $file) {
            if (!isset($patches[$project['name']])) {
              $patches[$project['name']] = array('project' => $project, 'files' => array());
            }
            // Test if filename matches expected format of a drupal.org/node/% argument.
            // Look for suffix '#comment-cid'.
            $suffixarray = array();
            $suffixarray = explode('#comment-', $file->name, 2);
            // Look for prefix 'projectname-'.
            $prefixarray = array();
            $prefixarray = explode('-', $suffixarray[0], 2);
            if (is_numeric($suffixarray[0]) && !$suffixarray[1]) { // Test for nid.patch.
              $file->cleanpath = $file->name;
            } elseif (is_numeric($suffixarray[0]) && is_numeric($suffixarray[1])) { // Test for nid#comment-cid.patch.
              $file->cleanpath = $file->name;
            } elseif (is_numeric($prefixarray[1]) && !$suffixarray[1]) { // Test for projectname-nid.patch.
              $file->cleanpath = $prefixarray[1];
            } elseif (is_numeric($prefixarray[1]) && is_numeric($suffixarray[1])) { // Test for projectname-nid#comment-cid.patch.
              $file->cleanpath = $prefixarray[1] . '#comment-' . $suffixarray[1];
            }
            if ($file->cleanpath) {
              $file->title = _civicactions_get_patch_title($file->cleanpath);
            }
            $patches[$project['name']]['files'][] = $file;
        }
      }
    }
    return theme('civicactions_patches', $patches);
  }
  drupal_set_message(t('Update Status must be enabled, <a href="@url">enable it now</a>', array('@url' => url('admin/build/modules'))), 'error');
  return '';
}

function _civicactions_get_project_path($project) {
  $project_path = '';
  foreach (array_keys($project['includes']) as $project_name) {
    $project_dir = drupal_get_path('module', $project_name);
    // Check for themes too.
    if (!$project_dir) {
      $project_dir = drupal_get_path('theme', $project_name);
    }
    if (!$project_path || strlen($project_dir) < strlen($project_path)) {
      $project_path = $project_dir;
    }
  }
  return $project_path;
}

function _civicactions_get_patch_title($nid) {
  // First try and find the patch title from the cache.
  if ($cache = cache_get('civicactions_patch_info', 'cache')) {
    $titles = unserialize($cache->data);
    if (isset($titles[$nid])) {
      return $titles[$nid];
    }
  }
  // The title's not in the cache, so get the page from d.o.
  $html = drupal_http_request('http://drupal.org/node/'. $nid);
  if ($html->code == 200) {
    // We got a valid page from d.o, find the title.
    if ($title = _civicactions_parse_title($html->data)) {
      // Save the title in a cache so we don't need to do this too often.
      if (!isset($titles)) {
        $titles = array();
      }
      $titles[$nid] = $title;
      cache_set('civicactions_patch_info', 'cache', serialize($titles), time() + 86400);
      return $title;
    }
  }
}

function _civicactions_parse_title($html) {
  if (preg_match(',<title>(.*)</title>,', $html, $matches)) {
    return trim(preg_replace('/\|\s*drupal\.org$/', '', $matches[1]));
  }
}

function theme_civicactions_patches($patches) {
  if (count($patches)) {
    foreach ($patches as $path => $patch) {
      $rows[] = array(
        theme('civicactions_patch_project', $patch['project']),
        theme('civicactions_patch_files', $patch['files'])
      );
    }
    return theme('table', array(t('Project'), t('Patches')), $rows);
  }
  return t('No patches found.');
}

function theme_civicactions_patch_project($project) {
  return l($project['name'], 'http://drupal.org/project/'. $project['name']);
}

function theme_civicactions_patch_files($files) {
  $items = array();
  foreach ($files as $file) {
      // Create link if filename matches expected format.
      if ($file->cleanpath) {
        $item = l($file->name, 'http://drupal.org/node/'. $file->cleanpath);
      } else {
        $item = t($file->name);
      }
    if (!empty($file->title)) {
      $item .= ': '. check_plain($file->title);
    }
    $items[] = $item;
  }
  return implode('<br/>', $items);
}
